.TH CLOCK TCL "" "TCL - Tool Command Language"
.ad b
.SH NAME
getclock, fmtclock, settimezone - TCL date and time commands.
.SH SYNOPSIS
.br
.B getclock
.br
.B fmtclock \fIclockval\fR [\fIformat\fR]
.br
.B settimezone \fItzspec\fR
.br
.SH DESCRIPTION
.PP
'@help: extend/getclock
.B getclock
.br
Return the current date and time as a system dependent integer value.  The
unit of the value is seconds, allowing calculations to be performed on it.
'@endhelp
.PP
'@help: extend/fmtclock
.B fmtclock \fIclockval\fR [\fIformat\fR]
.br
Convert a value returned by \fBgetclock\fR to human readable form.  The
\fIformat\fR argument is a string that describes how to format the date and
time.  Field descriptors consist of a "%" followed by field descriptor. All
other characters are copied into the result.  The available field descriptors
are:
.sp
.nf
.ft CW
    %% - Insert a %.
    %a - Abbreviated weekday name.
    %A - Full weekday name
    %b - Abbreviated month name.
    %B - Full month name.
    %d - Day of month (01 - 31).
    %D - Date as %m/%d/%y.
    %e - Day of month (1-31), no leading zeros.
    %h - Abbreviated month name.
    %H - Hour (00 - 23).
    %I - Hour (00 - 12).
    %j - Day number of year (001 - 366).
    %m - Month number (01 - 12).
    %M - Minute (00 - 59).
    %n - Insert a new line.
    %p - AM or PM.
    %r - Time as %I:%M:%S %p.
    %R - Time as %H:%M.
    %S - Seconds (00 - 59).
    %t - Insert a tab.
    %T - Time as %H:%M:%S.
    %U - Week number of year (01 - 52), Sunday is the first 
         day of the week.
    %w - Weekday number (Sunday = 0).
    %W - Week number of year (01 - 52), Monday is the first 
         day of the week.
    %x - Local specific date format.
    %X - Local specific time format.
    %y - Year within century (00 - 99).
    %Y - Year as ccyy (e.g. 1990)
    %Z - Time zone name.
.fi
.ft R
If format is not specified, "%a %b %d %H:%M:%S %Z %Y" is used.
'@endhelp
.PP
'@help: extend/settimezone
.B settimezone \fItzspec\fR
.br
Sets the time zone for \fBfmtclock\fR to use in formatting clock values.  
\fITzspec\fR is one of \fBlocal\fR or \fBGMT\fR.  This command is not available
in Unix implementations that do not have the tzset library routine.
'@endhelp
.SH INITIALIZATION
To initialize the clock commands in a custom TCL environment call:
.sp
.nf
.ft CW
    void Tcl_InitClock (interp)
           Tcl_Interp *interp;
.ft R
.fi
.SH BUGS
.PP
The if would be nice to support any timezone in the \fBsettimezone\fR command.
