.TH STRING TCL "" "TCL - Tool Command Language"
.ad b
.SH NAME
char, cindex, clength, crange, lowercase, ordinal, replicate, strcat, 
strequ, substr, translit, trim_left, trim_right, trim_string, uppercase - TCL
extended string commands.
.SH SYNOPSIS
.br
.B char \fInumber\fR
.br
.B cindex \fIstring index\fR
.br
.B clength \fIstring\fR
.br
.B crange \fIstring first last\fR
.br
.B lowercase \fIstring\fR
.br
.B ordinal \fIstring\fR
.br
.B replicate \fIstring count\fR
.br
.B strequ \fIstring1 string2\fR
.br
.B strcat \fIstrvar string\fR
.br
.B substr \fIstring first length\fR
.br
.B translit \fIinrange outrange string\fR
.br
.B trim_left \fIstring\fR [\fIchars\fR]
.br
.B trim_right \fIstring\fR [\fIchars\fR]
.br
.B trim_string \fIstring\fR [\fIchars\fR]
.br
.B uppercase \fIstring\fR
.br
.SH DESCRIPTION
.PP
'@help: string/char
.B char \fInumber\fR
.br
Converts the numeric value, number to an ascii character.  Number must be in 
the range 0 through 255.
'@endhelp
.PP
'@help: string/cindex
.B cindex \fIstring index\fR
.br
Returns the character indexed by \fIindex\fR (zero based) from \fIstring\fR.
This command is a shortcut for:
.nf
.ft CW
    index string index chars
.ft R
.fi
'@endhelp
.PP
'@help: string/clength
.B clength \fIstring\fR
.br
Returns the length of \fIstring\fR in characters.  
This command is a shortcut for:
.nf
.ft CW
    length string chars
.ft R
.fi
'@endhelp
.br
.PP
'@help: string/crange
.B crange \fIstring first last\fR
.br
Returns a range of characters from \fIstring\fR starting at \fIfirst\fR (zero
based) until \fIlast\fR.  The special keyword 'end' may be specified
for \fIlast\fR to indicate that the remainder of the string is to be 
extracted.  This command is a short cut for:
.nf
.ft CW
    range string first last chars
.ft R
.fi
'@endhelp
.PP
'@help: string/lowercase
.B lowercase \fIstring\fR
.br
Return \fIstring\fR with upper case characters mapped to lower case characters.
'@endhelp
.PP
'@help: string/ordinal
.B ordinal \fIstring\fR
.br
Returns the ordinal value of the first character in \fIstring\fR.
'@endhelp
.br
.PP
'@help: string/replicate
.B replicate \fIstring count\fR
.br
Returns \fIstring\fR replicated \fIcount\fR times.
'@endhelp
.PP
'@help: string/strcat
.B strcat \fIstrvar string\fR
.br
Append \fIstring\fR to the end of the variable \fIstrvar\fR.  If \fIstrvar\fR
does not exists, it is created.  The resulting value of the \fIstrvar\fR is
also returned as the result of the command.
'@endhelp
.PP
'@help: string/strequ
.B strequ \fIstring1 string2\fR
.br
Returns 1 if \fIstring1\fR is lexicographically equal to \fIstring2\fR.
This command is a shortcut for:
.nf
.ft CW
    [expr [string compare string1 string2]==0]
.ft R
.fi
'@endhelp
'@help: string/substr
.B substr \fIstring first length\fR
.br
Returns a range of characters from \fIstring\fR starting at \fIfirst\fR (zero
based) for \fIlength\fR characters.  The special keyword 'end' may be specified
for \fIlength\fR to indicate that the remainder of the string is to be 
extracted.
'@endhelp
.PP
'@help: string/translit
.B translit \fIinrange outrange string\fR
.br
Translate characters in \fIstring\fR matching characters in \fIinrange\fR
to the corresponding character in \fIoutrange\fR. \fIInrange\fR and 
\fIoutrange\fR may be list of characters or a range in the form 'A-M'.
.nf
.ft CW
    Example:
        translit a-z A-Z foobar
    returns 
        FOOBAR
.ft R
.if
'@endhelp
.PP
'@help: string/trim_left
.B trim_left \fIstring\fR [\fIchars\fR]
.br
Return \fIstring\fR with all leading characters that are in the string 
\fIchars\fR removed.  If \fIchars\fR is omitted, whitespaces (spaces, tabs, 
newlines and carriage returns) are removed.
'@endhelp
.PP
'@help: string/trim_right
.B trim_right \fIstring\fR [\fIchars\fR]
.br
Return \fIstring\fR with all trailing characters that are in the string 
\fIchars\fR removed.  If \fIchars\fR is omitted, whitespaces (spaces, tabs, 
newlines and carriage returns) are removed.
'@endhelp
.PP
'@help: string/trim_string
.B trim_string \fIstring\fR [\fIchars\fR]
.br
Return \fIstring\fR with all leading and trailing characters that are in the
string \fIchars\fR removed.  If \fIchars\fR is omitted, whitespaces (spaces, 
tabs, newlines and carriage returns) are removed.
'@endhelp
.PP
'@help: string/uppercase
.B uppercase \fIstring\fR
.br
Return \fIstring\fR with lower case characters mapped to upper case characters.
'@endhelp
.br
.SH INITIALIZATION
To initialize all the basic TCL string extension commands (except for
\fBtranslit\fR) in a custom TCL environment call:
.sp
.nf
.ft CW
    void TCL_InitString (interp)
           Tcl_Interp *interp;
.ft R
.fi
To initialize the \fBtranslit\fR command in a custom TCL environment call:
.sp
.nf
.ft CW
    void TCL_InitTranslit (interp)
           Tcl_Interp *interp;
.ft R
.fi
