.TH UNIXCMDS TCL "" "TCL - Tool Command Language"
.ad b
.SH NAME
chmod, env, execvp, exit, fork, getenv, getpid,
kill, putenv, signal, system, times, umask, unlink,
wait - TCL Unix access commands.
.SH SYNOPSIS
.B chmod \fImode\fR \fIfilename\fR
.br
.B env
.br
.B execvp \fIprog\fR [\fI"arg1...argN"\fR]
.br
.B exit \fIretcode\fR
.br
.B fork
.br
.B getenv \fIenvvar\fR [\fItclvar\fR]
.br
.B getpid
.br
.B kill [\fIsignal\fR] \fIprocessid\fR
.br
.B putenv \fIenvvar\fR \fIenvvalue\fR
.br
.B signal \fIsigname\fR [\fIaction\fR]
.br
.B system \fIcommand\fR
.br
.B times
.br
.B umask [\fIoctalmask\fR]
.br
.B unlink \fIpath\fR [\fIpath\fR..]
.br
.B wait
.br
.SH DESCRIPTION
.PP
'@help: extend/chmod
.B chmod \fImode\fR \fIfilename\fR
.br
Set permissions of \fIfilename\fR to \fImode\fR, which is an octal number or a
symbolic permissions as in the \fBchmod(C)\fR UNIX command.
'@endhelp
.PP
'@help: extend/env
.B env
.br
Return the environment in Tcl list format, as in:
.br
{{HOME /usr/karl} {EDITOR vi} {HZ 100} {TERM vt100} ...}
'@endhelp
.PP
'@help: extend/execvp
.B execvp \fIprog\fR [\fI"arg1...argN"\fR]
.br
Do an execvp, replacing the current program with \fIprog\fR and passing the
arguments \fI"arg1...argN"\fR.
'@endhelp
.PP
'@help: extend/exit
.B exit \fIretcode\fR
.br
Exit the current TCL process with a return code of \fIretcode\fR.
'@endhelp
.PP
'@help: extend/fork
.B fork
.br
Fork the current TCL process. Fork returns zero to the child process and the
process number of the child to the parent process.
'@endhelp
.PP
'@help: extend/getenv
.B getenv \fIenvvar\fR [\fItclvar\fR]
.br
Return the value of the environment variable specified by \fIvar\fR or return
an empty string if the specified variable was not found.  If \fItclvar\fR is 
specified, then the value of the environment variable is returned to
\fItclvar\fR and the command returns 1 if \fIenvvar\fR was found, 0 if it was 
not.
'@endhelp
.PP
'@help: extend/getpid
.B getpid
.br
Return the current process ID.
'@endhelp
.PP
'@help: extend/kill
.B kill [\fIsignal\fR] \fIprocessid\fR
.br
Send a signal to \fIprocessid\fR, if permitted.  \fISignal\fR, if present, is
the signal number or the symbolic name of the signal, see signal(S).  The
leading "SIG" is optional in the symbolic name.  The default is for
\fIsigno\fR is 2, SIGINT.
'@endhelp
.PP
'@help: extend/putenv
.B putenv \fIenvvar\fR \fIenvvalue\fR
.br
Add or change an environment variable.  The environment variable, \fIenvvar\fR
is assigned the value \fIenvvalue\fR.  This environment is then passed to all
new child processes.
'@endhelp
.PP
'@help: extend/signal
.B signal \fIsigname\fR [\fIaction\fR]
.br
Specify the action to take when a Unix signal occurs. \fISigname\fR is either
the symbolic or numeric Unix signal (the SIG prefix is optional). \fIAction\fR
is one of the following actions to be performed on receipt of the signal.  
If \fIaction\fR is omitted, \fBcatch\fR is assumed.
.br
.sp
    \fBdefault\fR - Terminate the process
.br
    \fBignore\fR - Ignore the signal.
.br
    \fBcatch\fR - Generate a catch-able TCL error.
'@endhelp
.PP
'@help: extend/system
.B system \fIcommand\fR
.br
Executes \fIcommand\fR via the system(3) call.  Differs from \fBexec\fI
because \fBsystem\fR doesn't return stdout as the result or the command and
system goes through the shell to provide wildcard expansion, redirection, etc,
as is normal from a sh command line.  Returns the exit code of the command.
'@endhelp
.PP
'@help: extend/times
.B times
.br
Return a list containing the process and child execution times in the form:
.br
{utime stime cutime cstime}
.br
See times(S).  The values are in milliseconds.
'@endhelp
.PP
'@help: extend/umask
.B umask [\fIoctalmask\fR]
.br
Sets file-creation mode mask to the octal value of \fIoctalmask\fR.
If \fIoctalmask\fR is omitted, the current mask is returned.
'@endhelp
.PP
'@help: extend/unlink
.B unlink \fIpath\fR [\fIpath\fR..]
.br
Delete the files specified by \fIpath\fR.
'@endhelp
.PP
'@help: extend/wait
.B wait
.br
Waits for an immediate child process to terminate or a signal to be received.
Returns a TCL list of three elements, the process id of the terminating 
process and the high and low part of the wait status.  The low part is the 
process exit code.  See wait(S).
'@endhelp
.SH INITIALIZATION
To initialize all of the Unix TCL access commands in a custom TCL environment 
call:
.sp
.nf
.ft CW
    void TCL_InitUnixCmds (interp)
           Tcl_Interp *interp;
.ft R
.fi
.SH BUGS
If multiple interpreters are running in the same process, the specified signal
actions are global to all interpreters and the one running when the signal
occurs is the one that traps it.
