/* 
 * chartype.c --
 *
 *      TCL string classification commands.
 *
 */

/* 
 * Copyright 1990 Karl Lehenbauer, Mark Diekhans, 
 *                Peter da Silva and Jordan Henderson.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  Karl Lehenbauer, Mark Diekhans,
 * Peter da Silva and Jordan Henderson make no representations
 * about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 */

#ifndef NOSCCSID
        static char *sccs_id = "@(#) string.c 2.2 10/2/90";
#endif

#include <sys/types.h>
#include <ctype.h>
#include <string.h>

#include "tcl.h"

/*
 *----------------------------------------------------------------------
 *
 * Tcl_CharClassCmd --
 *
 *      This function implements the 'ctype' command:
 *      ctype class string
 *
 *      Where class is one of the following:
 *        digit, xdigit, lower, upper, alpha, alnum,
 *        space, cntrl,  punct, print, graph, ascii
 *
 * Results:
 *       One or zero: Depending if all the characters in the string are of
 *       the desired class.
 *
 *----------------------------------------------------------------------
 */
int
  Tcl_CharClassCmd (clientData, interp, argc, argv)
    char        *clientData;
    Tcl_Interp  *interp;
    int          argc;
    char       **argv;
{ /* Tcl_CharClassCmd */
register char *Class;
register char *ScanPtr;

if (argc != 3)
  return Tcl_RetError (interp, "TCL", "bad # arg: %.50s class string", 
                       argv[0]);

/*
 * Select based on the first letter of the 'class' argument to chose the 
 * macro to test characters with.  In some cases another character must be
 * switched on to determine which macro to use.  This is gross, but better
 * we only have to do a string compare once to test if class is correct.
 */

Class = argv [1];
if ((Class [0] == 'a') && (Class [2] == 'n') && (strcmp (Class, "alnum") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isalnum (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'a') && (Class [2] == 'p') && (strcmp (Class, "alpha") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (! isalpha (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'a') && (Class [1] == 's') && (strcmp (Class, "ascii") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isascii (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'c') && (strcmp (Class, "cntrl") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!iscntrl (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'd') && (strcmp (Class, "digit") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isdigit (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'g') && (strcmp (Class, "graph") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isgraph (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'l') && (strcmp (Class, "lower") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!islower (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'p') && (Class [1] == 'r') && (strcmp (Class, "print") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isprint (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'p') && (Class [1] == 'u') && (strcmp (Class, "punct") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!ispunct (*ScanPtr))
      break;
    }
else
if ((Class [0] == 's') && (strcmp (Class, "space") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isspace (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'u') && (strcmp (Class, "upper") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isupper (*ScanPtr))
      break;
    }
else
if ((Class [0] == 'x') && (strcmp (Class, "xdigit") == 0))
  for (ScanPtr = argv [2]; *ScanPtr != 0; ScanPtr++) {
    if (!isxdigit (*ScanPtr))
      break;
    }
else
  return Tcl_RetError (interp, "TCL", 
                       "unrecognized class specification: %.50s", Class);

/*
 * Return true or false, depending if the end was reached.  Always return 
 * false for a null string.
 */
interp->result [0] = (*ScanPtr == 0 && ScanPtr != argv [2]) ? '1' : '0';
interp->result [1] = 0;
return TCL_OK;

} /* Tcl_CharClassCmd */

/*
 *----------------------------------------------------------------------
 *
 *  Tcl_InitCharType --
 *
 *      Initialize the ctype Tcl character classification command.
 *
 *----------------------------------------------------------------------
 */
void
  Tcl_InitCharType (interp)
    Tcl_Interp *interp;
{ /* Tcl_InitCharType */

Tcl_CreateCommand (interp, "ctype", Tcl_CharClassCmd,
                   (ClientData)NULL, (void (*)())NULL);

} /* Tcl_InitCharType */

