
/* 
 * Copyright 1990 Karl Lehenbauer, Mark Diekhans, 
 *                Peter da Silva and Jordan Henderson.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  Karl Lehenbauer, Mark Diekhans,
 * Peter da Silva and Jordan Henderson make no representations
 * about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 */

#ifndef NOSCCSID
        static char *sccs_id = "%M% %I% %H%";
#endif

/*
        custom_info name
                returns the name of the custom version of Tcl

        custom_info longname
                returns a one-line description of the custom version of Tcl

        custom_info patchlevel
                returns the patch level of the current custom version

*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/times.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tcl.h"

extern char *custom_name;
extern char *custom_longname;
extern char *patchlevel;
extern char *tcl_defaultfile;

int
cmdcustom_info(clientData, interp, argc, argv)
    char *clientData;
    Tcl_Interp *interp;
    int argc;
    char **argv;
{
        char *s;
        int len;

        if (argc != 2)
                return Tcl_RetError (interp, "TCL", "bad # arg: %.50s option", 
                                     argv[0]);

        s = argv[1];
        len = strlen(s);

        if ((*s == 'n') && (strncmp(s, "name", len) == 0)) {
                strcpy(interp->result, custom_name);
                return TCL_OK;
        }

        if ((*s == 'l') && (strncmp(s, "longname", len) == 0)) {
                strcpy(interp->result, custom_longname);
                return TCL_OK;
        }

        if ((*s == 'd') && (strncmp(s, "defaultfile", len) == 0)) {
                strcpy(interp->result, tcl_defaultfile);
                return TCL_OK;
        }

        if ((*s == 'p') && (strncmp(s, "patchlevel", len) == 0)) {
                sprintf(interp->result, "%d", patchlevel);
                return TCL_OK;
        }

        return Tcl_RetError (interp, "TCL", 
                        "bad option (%s): must be %s",
                        s, "defaultfile, name, longname, or patchlevel");
}

Tcl_InitCustom(interp)
Tcl_Interp *interp;
{
    Tcl_CreateCommand(interp, "custom_info", cmdcustom_info, (ClientData)NULL, (void (*)())NULL);

}

