/* 
 * Copyright 1990 Karl Lehenbauer, Mark Diekhans, 
 *                Peter da Silva and Jordan Henderson.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  Karl Lehenbauer, Mark Diekhans,
 * Peter da Silva and Jordan Henderson make no representations
 * about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 */

#ifndef NOSCCSID
        static char *sccs_id = "%M% %I% %H%";
#endif


#include <sys/types.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>

#include "tcl.h"

extern int signal_received;  /* Can't move to interp due to SignalTrap */
extern char *signal_name;

/*
 * Signal name table maps name to number.
 */

#define SIG_NAME_MAX 7

struct {char *name;
        short num;
       } SigNameTable [] = {
    "SIGHUP",     SIGHUP,
    "SIGINT",     SIGINT,
    "SIGQUIT",    SIGQUIT,
    "SIGILL",     SIGILL,
    "SIGTRAP",    SIGTRAP,
    "SIGIOT",     SIGIOT,
#ifdef SIGABRT
    "SIGABRT",    SIGABRT,
#endif
    "SIGEMT",     SIGEMT,
    "SIGFPE",     SIGFPE,
    "SIGKILL",    SIGKILL,
    "SIGBUS",     SIGBUS,
    "SIGSEGV",    SIGSEGV,
    "SIGSYS",     SIGSYS,
    "SIGPIPE",    SIGPIPE,
    "SIGALRM",    SIGALRM,
    "SIGTERM",    SIGTERM,
    "SIGUSR1",    SIGUSR1,
    "SIGUSR2",    SIGUSR2,
    "SIGCLD",     SIGCLD,
#ifdef SIGPWR
    "SIGPWR",     SIGPWR,
#endif
#ifdef SIGPOLL
    "SIGPOLL",    SIGPOLL,
#endif
#ifdef SIGSTOP
    "SIGSTOP",    SIGSTOP,
    "SIGTSTP",    SIGTSTP,
    "SIGCONT",    SIGCONT,
    "SIGTTIN",    SIGTTIN,
    "SIGTTOU",    SIGTTOU,
#endif
    NULL,         -1};

#ifdef BSD
#   define SIG_PROC_TYPE int
#else
#   define SIG_PROC_TYPE void
#endif
/*
 *----------------------------------------------------------------------
 *
 * SigNumToName --
 *     Converts a UNIX signal number to a name, returns NULL if not found.
 *
 *----------------------------------------------------------------------
 */
char*
SigNumToName (SigNum)
    int SigNum;
{
int idx;

for (idx = 0; SigNameTable[idx].name != NULL; idx++)
    if (SigNameTable[idx].num == SigNum)
        break;

return SigNameTable [idx].name;
}

/*
 *----------------------------------------------------------------------
 *
 * SigNameToNum --
 *     Converts a UNIX signal name to its number, returns -1 if not found.
 *     the name may be upper or lower case and may optionally have the 
 *     leading "SIG" omitted.
 *
 *----------------------------------------------------------------------
 */
int
SigNameToNum (SigName)
    char *SigName;
{
char  SigNameUp [SIG_NAME_MAX+1];  /* Upshifted signal name */
int   idx;

/*
 * Copy and upshift requested name.
 */

if (strlen (SigName) > SIG_NAME_MAX)
    return -1;   /* Name too long */
UpShift (SigNameUp, SigName);

for (idx = 0; SigNameTable[idx].name != NULL; idx++)
    if ((strcmp (SigNameUp, SigNameTable[idx].name) == 0) ||
        (strcmp (SigNameUp, &(SigNameTable[idx].name[3])) == 0))
        break;

return SigNameTable [idx].num;
}

/*
 *----------------------------------------------------------------------
 *
 * SignalTrap --
 *     Trap handler for UNIX signals.  Sets a flag indicating that the
 *     trap has occured, saves the name and rearms the trap.  The flag
 *     will be seen by the interpreter when its safe to trap.
 *
 *----------------------------------------------------------------------
 */
SIG_PROC_TYPE
SignalTrap (SigNum)
    int SigNum;
{
signal_name = SigNumToName (SigNum);
signal_received = 1;
signal (SigNum, SignalTrap);
}

/*
 *----------------------------------------------------------------------
 *
 * CmdSignal --
 *     Implements the TCL signal command:
 *         signal signame [action]
 *     See the unixcmds(TCL) manual page.
 *
 * Results:
 *      Standard TCL results, may return the UNIX system error message.
 *
 * Side effects:
 *
 *----------------------------------------------------------------------
 */
int
CmdSignal (clientData, interp, argc, argv)
    char       *clientData;
    Tcl_Interp *interp;
    int         argc;
    char      **argv;
{
int            SigNum;
char          *SigName;
SIG_PROC_TYPE  (*ActionFunc)();

if ((argc < 2) || (argc > 3))
    return Tcl_RetError (interp, "TCL", 
                         "bad # arg: %.50s signame [action]", argv[0]);

SigName = argv [1];
if (StrToInt (SigName, 10, &SigNum))
    SigName = SigNumToName (SigNum);
else
    SigNum = SigNameToNum (SigName);

if ((SigName == NULL) || (SigNum < 0))
    return Tcl_RetError (interp, "TCL", "invalid signal name", argv[0]);

if (argc == 3)
    {
    if (strcmp (argv[2], "default") == 0)
        ActionFunc = SIG_DFL;
    else if (strcmp (argv[2], "ignore") == 0)
        ActionFunc = SIG_IGN;
    else if (strcmp (argv[2], "catch") == 0)
        ActionFunc = SignalTrap;
    else
        return Tcl_RetError (interp, "TCL", 
                             "invalid signal action specified (%s)", argv[2]);
    }
else
    ActionFunc = SignalTrap;

signal (SigNum, ActionFunc);
return TCL_OK;
}
