/*
 * cftime for XENIX and BSD:
 * From: bdb@becker.UUCP (Bruce D. Becker)
 * Newsgroups: alt.sources.d,comp.sources.wanted,comp.sources.d,comp.lang.c
 * Subject: Re: System V-compatible ctime(3)???
 * Message-ID: <34765@becker.UUCP>
 * Date: 1 Sep 90 14:09:27 GMT
 * References: <862@iiasa.UUCP>
 * Organization: G. T. S., Toronto, Ontario, Canada
 */

#ifdef BSD
#include	<sys/time.h>
#else
#include	<time.h>
#endif

#ifdef XENIX286
extern char *tzname[];
#endif

static char	*amon[] = {	"Jan","Feb","Mar","Apr","May","Jun",
				"Jul","Aug","Sep","Oct","Nov","Dec" };
static char	*lmon[] = {	"January","February","March",
				"April","May","June",
				"July","August","September",
				"October","November","December" };
static char	*awkd[] = {	"Sun","Mon","Tue","Wed","Thu","Fri","Sat" };
static char	*lwkd[] = {	"Sunday","Monday","Tuesday","Wednesday",
				"Thursday","Friday","Saturday" };

int	ascftime(buf, fmt, tm)
char 	  *buf, *fmt;
struct tm *tm;
{

	char		c, *ctmp, *bb;
	int		i;

	if (fmt == (char *)0 || *fmt == '\0') fmt = "%a %b %d %T %Z %Y";
	bb = buf;
	while(c = *(fmt++)) {
		if (c != '%') *(buf++) = c;
		else switch(c = *(fmt++)) {
		case 'a':
			ctmp = awkd[tm->tm_wday];
			while (*(buf++) = *(ctmp++));
			--buf;
			break;
		case 'A':
			ctmp = lwkd[tm->tm_wday];
			while (*(buf++) = *(ctmp++));
			--buf;
			break;
		case 'b':
		case 'h':
			ctmp = amon[tm->tm_mon];
			while (*(buf++) = *(ctmp++));
			--buf;
			break;
		case 'B':
			ctmp = lmon[tm->tm_mon];
			while (*(buf++) = *(ctmp++));
			--buf;
			break;
		case 'd':
			sprintf(buf, "%02d", tm->tm_mday);
			buf += 2;
			break;
		case 'D':
		case 'x':
			sprintf(buf, "%02d/%02d/%02d",
			tm->tm_mon+1, tm->tm_mday, tm->tm_year);
			buf += 8;
			break;
		case 'e':
			sprintf(buf, "%2d", tm->tm_mday);
			buf += 2;
			break;
		case 'H':
			sprintf(buf, "%02d", tm->tm_hour);
			buf += 2;
			break;
		case 'I':
			i = tm->tm_hour % 12;
			sprintf(buf, "%02d", i?i:12);
			buf += 2;
			break;
		case 'j':
			sprintf(buf, "%03d", tm->tm_yday+1);
			buf += 3;
			break;
		case 'm':
			sprintf(buf, "%2d", tm->tm_mon+1);
			buf += 2;
			break;
		case 'M':
			sprintf(buf, "%02d", tm->tm_min);
			buf += 2;
			break;
		case 'n':
			*(buf++) = '\n';
			break;
		case 'r':
			i = tm->tm_hour % 12;
			sprintf(buf, "%02d:%02d:%02d ",
			i?i:12, tm->tm_min, tm->tm_sec);
			buf += 9;
		case 'p':
			if (tm->tm_hour <12 ) *(buf++) = 'A';
			else *(buf++) = 'P';
			*(buf++) = 'M';
			break;
		case 'R':
			sprintf(buf, "%02d:%02d",
			tm->tm_hour, tm->tm_min);
			buf += 5;
			break;
		case 'S':
			sprintf(buf, "%2d", tm->tm_sec);
			buf += 2;
			break;
		case 't':
			*(buf++) = '\t';
			break;
		case 'T':
		case 'X':
			sprintf(buf, "%02d:%02d:%02d",
			tm->tm_hour, tm->tm_min, tm->tm_sec);
			buf += 8;
			break;
		case 'U':
			sprintf(buf, "%2d", ((tm->tm_yday+7-tm->tm_wday)/7)+1);
			buf += 2;
			break;
		case 'w':
			sprintf(buf, "%1d", tm->tm_wday);
			buf += 1;
			break;
		case 'W':
			sprintf(buf, "%2d", ((tm->tm_yday+6-tm->tm_wday)/7)+1);
			buf += 2;
			break;
		case 'y':
			sprintf(buf, "%2d", tm->tm_year);
			buf += 2;
			break;
		case 'Y':
			if ((i = tm->tm_year) < 70) i += 100;
			i += 1900; 
			sprintf(buf, "%4d", i);
			buf += 4;
			break;
		case 'Z':
#ifdef BSD
			{
				struct timeval	tp;
				struct timezone	tz;

				gettimeofday(&tp, &tz);
				ctmp = (char *)timezone(tz.tz_minuteswest,
					tm->tm_isdst);
			}
#else
			tzset();
			i = (tm->tm_isdst)? 1: 0;
			ctmp = tzname[i];
#endif
			while (*(buf++) = *(ctmp++));
			--buf;
			break;
		case '%':
			*(buf++) = c;
			break;
		default:
			sprintf(buf, "bad format character - %c", c);
			return (-1);
		}
	}
	return (buf-bb);
}

