#
# gen_dfault.tcl - create default path file if necessary
#
if {[length $argv] != 3} {
    puts stderr "usage: tcl -f gen_dfault.tcl default_file source_path init_file
}

set defaultfile [index $argv 0][info tclversion]
set tclpath [index $argv 1]
set initfile [index $argv 2]

if [file $defaultfile exists] {
	puts stderr "$defaultfile already exists -- I am leaving it alone..."
	exit 0
}

if ![file [file $defaultfile dirname] writable] {
	puts stderr "Can't create $defaultfile -- directory is not writable"
	puts stderr " Please reinstall with correct permissions or rebuild"
	puts stderr " Tcl to select a default file where the directory path"
	puts stderr " you specify is writable by you."
	puts stderr ""
	puts stderr "Tcl will still be runnable from the current directory,"
	puts stderr "but maybe not any others..."
	puts stderr ""
	exit 1
}

set fp [open $defaultfile w]

puts $fp "# generated by gen_default.tcl on [fmtclock [getclock]]"
puts $fp ""
puts $fp "# gen_default will not overwrite this if it exists, so you"
puts $fp "# can edit it to taste"
puts $fp ""
puts $fp "# set TCLINIT to specified file unless TCLINIT env var is set"
puts $fp "set_default TCLINIT $initfile"
puts $fp ""
puts $fp "# set TCLPATH to specified file unless TCLPATH env var is set"
puts $fp "set_default TCLPATH $tclpath"
puts $fp ""

puts stderr "generated $defaultfile"
close $fp
exec chmod a+r,go-w $defaultfile
exit 0
