#
# start of packages code
#

proc put_lib_line {outfp package libfile where endwhere autoprocs} {
	puts $outfp [format "defpackage %s %s %s %s %s" $package $libfile $where [expr {$endwhere - $where - 1}] $autoprocs]
}

global package_myscancontext
set oldscancontext [current_scancontext]
set package_myscancontext [create_scancontext]

match "^package " {
    set size [length $matchInfo(line)]
    if {$size < 2} {
	error [format "invalid package header \"%s\"" $matchInfo(line)]
    }
    if {$def_in_progress} {
	put_lib_line $outfp $package_name $libname $package_where $matchInfo(offset) $autoprocs
    }
    set def_in_progress 1
    set package_name [index $matchInfo(line) 1]
    set package_where $matchInfo(offset)
    set autoprocs [range $matchInfo(line) 2 end]
}

select_scancontext $oldscancontext
unset oldscancontext

proc create_lib_index {outfp libname} {
    global matchInfo
    global package_myscancontext

    set old_scancontext [current_scancontext]
    select_scancontext $package_myscancontext
    set def_in_progress 0
    set fp [open $libname r]
    scan_file $fp
    put_lib_line $outfp $package_name $libname $package_where [tell $fp] $autoprocs
    close $fp
    select_scancontext $old_scancontext
}

proc packages {} {
    global packages
    list_array_keys packages
}

proc autoprocs {} {
    global autoload_array
    list_array_keys autoload_array
}

proc build_package_index {packagename} {
    set indexname [file $packagename root].tndx
    set indexfp [open $indexname w]
    create_lib_index $indexfp $packagename
    close $indexfp
}

proc build_package_indexes {} {
    global TCL_PATHLIST
    set unwritable 0
    foreach dir $TCL_PATHLIST {
	if [strequ $dir "."] {set dir [pwd]}
	foreach pkg_file [globok $dir/*.tlib] {
	    if {[file $dir writable] || [file $pkg_file writable]} {
		puts stdout "building index for $pkg_file..." no_nl
		build_package_index $pkg_file
		puts stdout "ok"
	    } else {
		set indexname [file $packagename root].tndx
		if {![file $indexname exists] || ([file $indexname modify_time] < [file $pkg_file modify_time])} {
		    set unwritable 1
		}
	    }
	}
    }
    if $unwritable {
	puts stdout "WARNING - at least one package file index or directory doesn't exist or is"
	puts stdout "          out of date and you have insufficient permissions to fix it..."
    }
}

#
# package and defpackage have been moved to TclInit.tcl
#
