#
# set up an entry in your usenet news sys file to batch
# articles for groups you want to archive (:F:-style to
# batch),
#
# then do "tcl -f archive.tcl" and it'll cat your articles
# to files in subdirectories it'll create corresponding to
# the newsgroup name (edit the 'cd' below to cd to the top
# level of your archive directory).  the file it's catted
# into is yearmonthday, as in 19900825.
#
# Also an index file of the form Index19900825 is created at the
# top level that contains several useful fields from the newsgroup
# RFC-822-style header lines, plus a new Location field is created
# which contains the archive pathname, location within the archive
# file and length of the message, for use later by yet-unwritten
# searching software

set datepipe [open "|date +%Y%m%d" r]
set datefile [gets $datepipe]
close $datepipe

cd /archive/newscan

set tmpfile /usr/tmp/newscan[getpid]
exec mv /usr/spool/batch/newscan $tmpfile

set userlistfile UserList

set indexfile Index$datefile

set artfp [open $tmpfile r]

set userfp [open $userlistfile a]

set indexfp [open $indexfile a]

set verbose 1

while {[gets $artfp article]} {
	if {$verbose} {
		print [format "article %s\n" $article]
	}
	set artprop [prop open $article]
	puts $userfp [prop get $artprop From]

	set newsgroup [index [split , [prop get $artprop Newsgroups]] 0]

	set archivepath [join / [split . $newsgroup]]
	set archivename [join / [concat $archivepath $datefile]]

	if {$verbose} {
		print [format "archive to %s\n" $archivename]
	}
	mkpath $archivepath
	set archivefp [open $archivename a]
	set whence [tell $archivefp]

	set articlefp [open $article r]
	copy_open_files $articlefp $archivefp
	set whence_after [tell $archivefp]
	close $articlefp $archivefp

	puts $indexfp [concat From: [prop get $artprop From]]
	puts $indexfp [concat Newsgroups: [prop get $artprop Newsgroups]]
	puts $indexfp [concat Subject: [prop get $artprop Subject]]
	puts $indexfp [concat Message-ID: [prop get $artprop Message-ID]]
	puts $indexfp [concat Location: $archivename $whence [expr {$whence_after - $whence}]]
	puts $indexfp ""

	prop close $artprop
}

close $artfp $userfp $indexfp

exec rm $tmpfile 
